<?php
/**
 * class-woogrouppricing.php
 *
 * Copyright (c) Antonio Blanco http://www.blancoleon.com
 *
 * This code is released under the GNU General Public License.
 * See COPYRIGHT.txt and LICENSE.txt.
 *
 * This code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * This header and all notices must be kept intact.
 *
 * @author Antonio Blanco
 * @package woogrouppricing
 * @since woogrouppricing 1.0.0
 */

/**
 * WooGroupPricing class
 */
class WooGroupPricing {

	public static function init() {
	
		add_filter('woocommerce_get_price', array( __CLASS__, 'woocommerce_get_price' ), 10, 2);
		
		add_action('woocommerce_product_write_panel_tabs', array(__CLASS__,'woocommerce_product_write_panel_tabs') );
		add_action('woocommerce_product_write_panels', array(__CLASS__,'woocommerce_product_write_panels') );
		add_action('woocommerce_process_product_meta', array(__CLASS__,'woocommerce_process_product_meta') );

		if ( get_option( "wgp-displayoriginal", 0 ) == 1 ) {
			add_filter( 'woocommerce_get_price_html', array(__CLASS__,'woocommerce_get_price_html'), 100, 2 );
		}
	}
	
	public static function woocommerce_get_price ( $price, $product ) {
		global $post, $woocommerce;

		$baseprice = $price;
		$result = $baseprice;
		
		if ( ($post == null) || !is_admin() ) {
		
			if ( $product->is_type( 'variation' ) ) {
				$commission = WGP_Variations_Admin::get_commission( $product, $product->variation_id );
			} else {
				$commission = self::get_commission( $product );
			}
			
			if ( $commission ) {

				$baseprice = apply_filters( 'woo_group_pricing_baseprice', $product->get_regular_price(), $product );

				if ( $product->get_sale_price() != $baseprice && $product->get_sale_price() == $product->price ) {
					if ( get_option( "wgp-baseprice", "regular" )=="sale" ) {
						$baseprice = $product->get_sale_price();
					}
				}
				$product_price = apply_filters( 'woo_group_pricing_product_price', $baseprice, $product );
				
				$type = get_option( "wgp-method", "rate" );
				$result = 0;
				if ($type == "rate") {
					// if rate and price includes taxes
					if ( $product->is_taxable() && get_option('woocommerce_prices_include_tax') == 'yes' ) {
						$_tax       = new WC_Tax();
						$tax_rates  = $_tax->get_shop_base_rate( $product->tax_class );
						$taxes      = $_tax->calc_tax( $baseprice, $tax_rates, true );
						$product_price      = $_tax->round( $baseprice - array_sum( $taxes ) );
					}

					$result = self::bcmul($product_price, $commission, WOO_GROUP_PRICING_DECIMALS);
					
					if ( $product->is_taxable() && get_option('woocommerce_prices_include_tax') == 'yes' ) {
						$_tax       = new WC_Tax();
						$tax_rates  = $_tax->get_shop_base_rate( $product->tax_class );
						$taxes      = $_tax->calc_tax( $result, $tax_rates, false ); // important false
						$result      = $_tax->round( $result + array_sum( $taxes ) );
					}
				} else {
					$result = self::bcsub($product_price, $commission, WOO_GROUP_PRICING_DECIMALS);
				}
			}
		}

		return $result;
	}

	public static function woocommerce_product_write_panel_tabs() {

		echo '<li class="groups_pricing_tab general_options"><a href="#groups_pricing_data">' . __( 'Groups Pricing', WOO_GROUP_PRICING_DOMAIN ) . '</a></li>';

	}

	public static function woocommerce_product_write_panels() {
		global $post;
	
		global $post;
		if( function_exists('get_product') ){
			$product = get_product( $post->ID );
			if( $product->is_type( 'variable' ) ){
				// do something with external products
				echo '<div id="groups_pricing_data" class="panel woocommerce_options_panel">';
				echo "<p>In Variable Products these fields are in 'Variations' tabs.</p>";
				echo '</div>';
				return;
			}
		}
		$groups = self::get_all_groups();
		if ( $groups ) {
			$pricing_options = array();
			foreach ( $groups as $group ) {
				$pricing_options['value_' . $group->group_id] = get_post_meta($post->ID, 'groups_pricing_value_' . $group->group_id, true);
			}
		}
		?>
		<div id="groups_pricing_data" class="panel woocommerce_options_panel">
			<div class="options_group">
				<p class="description">
					<?php echo __( 'Leave empty if no custom group discount should be applied to this product (default setting).', WOO_GROUP_PRICING_DOMAIN ); ?>
				</p>
			</div>
			
			<div class="options_group custom_tab_options">
				<?php 
				foreach ( $groups as $group ) {
				?>
				<p class="form-field">
					<label><?php echo $group->name; ?>:</label>
					<input type="text" class="short" name="groups_pricing_value_<?php echo $group->group_id;?>" value="<?php echo @$pricing_options['value_' . $group->group_id]; ?>" />
				</p>
				<?php
				}
				?>
			</div>
		</div>
	<?php
	}

	public static function woocommerce_process_product_meta( $post_id ) {
		$groups = self::get_all_groups();
		if ( $groups ) {
			foreach ( $groups as $group ) {
				if ( isset($_POST['groups_pricing_value_' . $group->group_id]) && ( $_POST['groups_pricing_value_' . $group->group_id] !== "" ) ) {
					update_post_meta( $post_id, 'groups_pricing_value_' . $group->group_id, ( isset($_POST['groups_pricing_value_' . $group->group_id]) && ( $_POST['groups_pricing_value_' . $group->group_id] !== "" ) ) ? trim(str_replace(",", ".", $_POST['groups_pricing_value_' . $group->group_id])) : '' );
				} else {
					delete_post_meta( $post_id, 'groups_pricing_value_' . $group->group_id );
				}
			}
		}
	}

	public static function get_commission ( $product, $group = null ) {
		global $post, $woocommerce;

		$user_id = get_current_user_id();
		$user_groups = self::get_user_groups ( $user_id );
		$discount = 0;
		if ( sizeof( $user_groups ) > 0 ) {
			if ( isset( $group ) ) {
				$first_group = $group;
			} else {
				$first_group = self::get_user_group( $user_groups, $product );
			}

			$custom = "";
			if ( $product !== null ) {
				// Product - custom discount ...
				$custom = get_post_meta( $product->id, 'groups_pricing_value_' . $first_group->group_id, true);

				// Category - custom discount ....
				if ( $custom !== "" ) {
					$discount = $custom;
				} else {
					$categories = wp_get_post_terms( $product->id, 'product_cat',array('fields'=>'ids') );
					if ( sizeof( $categories ) > 0 ) {
						$max_cat_id = null;
						$max_cat_discount = 0;
						foreach ( $categories as $cat_id ) {
							$cat_discount = get_woocommerce_term_meta($cat_id, 'groups_pricing_value_' . $first_group->group_id, true);
							if ( $cat_discount !== "" ) {
								if ( $cat_discount > $max_cat_discount ) {
									$max_cat_discount = $cat_discount;
									$max_cat_id = $cat_id;
								}
							}
						}
						if ( $max_cat_id !== null ) {
							$custom = $max_cat_discount;
						}
					}
				}
			}
			// general discount ....
			if ( $custom !== "" ) {
				$discount = $custom;
			} else {
				if ( get_option( "wgp-" . $first_group->group_id, "-1" ) !== "-1" ) {
					$discount = get_option( "wgp-" . $first_group->group_id, "" );
				}
			}

			if ( $discount == "" ) {
				if ( get_option( "wgp-ifseveral", false ) == "lower" ) {
					$discount = 1;
				} else {
					$discount = 0;
				}
			}
			$method = get_option( "wgp-method", "rate" );
			if ( $method == "rate" ) {
				$discount = self::bcsub ( 1, $discount, WOO_GROUP_PRICING_DECIMALS );
				// for security reasons, set 0
				if ( $discount < 0 ) {
					$discount = 0;
				}
			}
		}

		return $discount;
	}

	public static function woocommerce_get_price_html( $price, $product ){
		$result = $price;
	
		if ( get_option( "wgp-baseprice", "regular" ) == "sale" ) {
			$price_key = '_price';
		} else {
			$price_key = '_regular_price';
		}
	
		if ( $product->product_type == 'variable' ) {
	
			$children = $product->get_children(true);
	
			$original_prices = array();
			$commission = 0;
			foreach ( $children as $child ) {
				$original_prices[] = get_post_meta( $child, $price_key, true );
				if ( $commission == 0 ) {
					$commission = WGP_Variations_Admin::get_commission( $product, $child );
				}
			}
	
			if ( $commission > 0 ) {
				$min_price = min( $original_prices );
				$max_price = max( $original_prices );
				$original_price = $min_price !== $max_price ? sprintf( _x( '%1$s–%2$s', 'Price range: from-to', 'woocommerce' ), wc_price( $min_price ), wc_price( $max_price ) ) : wc_price( $min_price );
			}
		} else {
			$commission = WooGroupPricing::get_commission( $product );
			if ( $commission ) {
				$original_price = woocommerce_price(get_post_meta( $product->id, $price_key, true ));
			}
		}
	
		if ( $commission ) {
			$result = apply_filters( 'wwo_group_pricing_display_original_prices', __( 'Was:', WOO_GROUP_PRICING_DOMAIN ) . $original_price . __( " Now:", WOO_GROUP_PRICING_DOMAIN ) . $price, $original_price, $price );
		}
		return $result;
	}


	/* Function relating to Groups */

	public static function get_all_groups (  ) {
		global $wpdb;

		$groups_table = _groups_get_tablename( 'group' );

		return $wpdb->get_results( "SELECT * FROM $groups_table ORDER BY name" );

	}

	/**
	 * Get all user groups.
	 * @param int $user_id
	 * @return array
	 */
	public static function get_user_groups( $user_id ) {
		global $wpdb;
	
		$groups_table = _groups_get_tablename( 'group' );
		$result = array();
		if ( $groups = $wpdb->get_results( "SELECT * FROM $groups_table ORDER BY group_id DESC" ) ) {
			foreach( $groups as $group ) {
				$is_member = Groups_User_Group::read( $user_id, $group->group_id ) ? true : false;
				if ( $is_member ) {
					$result[] = $group;
				}
			}
		}
		return  $result;
	}

	/**
	 * Get the group applied to an user.
	 * @param array $groups
	 * @return group_id or null if an error occurs.
	 */
	public static function get_user_group( $groups = array(), $product ) {
		$result = null;
		
		if ( sizeof( $groups ) > 0 ) {
			$option = get_option( "wgp-ifseveral", "first" );

			switch ( $option ) {
				case 'higher':
					$result = null;
					$commission = null;
					$method = get_option( "wgp-method", "rate" );
					foreach ( $groups as $group ) {
						$temp_com = self::get_commission( $product, $group );
						if ( $method == "rate" ) { // if rate: higher discount is lower commission
							if ( ( $commission == null ) || ( $temp_com < $commission ) ) {
								$commission = $temp_com;
								$result = $group;
							}
						} else {
							if ( ( $commission == null ) || ( $temp_com > $commission ) ) {
								$commission = $temp_com;
								$result = $group;
							}
						}
					}
					break;
				case 'lower':
					$result = null;
					$commission = null;
					$method = get_option( "wgp-method", "rate" );
					foreach ( $groups as $group ) {
						$temp_com = self::get_commission( $product, $group );
						if ( $method == "rate" ) { // if rate: lower discount is higher commission
							if ( ( $commission == null ) || ( $temp_com > $commission ) ) {
								$commission = $temp_com;
								$result = $group;
							}
						} else {
							if ( ( $commission == null ) || ( $temp_com < $commission ) ) {
								$commission = $temp_com;
								$result = $group;
							}
						}
					}
					break;
				case 'last':
					$result = $groups[sizeof($groups)-1];
					break;
				default:
				case 'first':
					$result = $groups[0];
					break;
			}
		}
		return $result;
	}

	/* Math functions */

	public static function bcmul( $data1, $data2, $prec = 0 ) {
		$result = 0;
		if ( function_exists('bcmul') ) {
			$result = bcmul( $data1, $data2, $prec );
		} else {
			$value = $data1 * $data2;
			if ($prec) {
				$result = round($value, $prec);
			}
		}
		return $result;
	}

	public static function bcsub( $data1, $data2, $prec = 0 ) {
		$result = 0;
		if ( function_exists('bcsub') ) {
			$result = bcsub( $data1, $data2, $prec );
		} else {
			$value = $data1 - $data2;
			if ($prec) {
				$result = round($value, $prec);
			}
		}
		return $result;
	}

}
WooGroupPricing::init();